/***************************************************************************
 *   Copyright (C) 2002 by Christopher TenHarmsel                          *
 *   tenharmsel@users.sourceforge.net                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _CARDPROFILE_H_
#define _CARDPROFILE_H_

#include <qstring.h>

class KConfig;

/**
 * Stores a configuration for the wireless card (ESSID, WEP Key, etc)
 * Christopher TenHarmsel
 **/
class CardProfile
{
public:
	/** Default constructor */
	CardProfile();

	/** Copy Constructor */
	CardProfile( const CardProfile &oldProfile );
		
	/** Constructor with values, sets the name to the same as the SSID
	 * @param ssid The desired SSID
	 * @param wepEnabled Whether or not to enable WEP
	 * @param wepKey The desired WEP key
	 * @param mode The Mode (CardProfile::MODE_ADHOC or
	 * CardProfile::MODE_INFRASTRUCTURE)
	 */
	CardProfile( const QString &ssid, bool wepEnabled, const QString &wepKey,
		     int mode);
		
	/** Constructor with values
	 * @param name The name of this profile
	 * @param ssid The desired SSID
	 * @param wepEnabled Whether or not to enable WEP
	 * @param wepKey The desired WEP key
	 * @param mode The Mode (CardProfile::MODE_ADHOC or
	 * CardProfile::MODE_INFRASTRUCTURE)
	 */
	CardProfile( const QString &name, const QString &ssid, bool wepEnabled,
		     const QString &wepKey, int mode);

	/** Destructor */
	~CardProfile();

	/**
	 * Assignment operator
	 */
	CardProfile& operator=(const CardProfile& copyCard);
		
	/** Accessor method for the SSID
	 * @return The SSID
	 */
	QString SSID() const { return _SSID; }
    
	/** Accessor method for the WEP Key
	 * @return The WEP Key
	 */
	QString WEPKey() const { return _WEPKey; }
		
	/** Accessor method for the mode
	 * @return The Mode
	 */
	int mode() const { return _Mode; }
		
	/** Accessor method for the WEP Enabled flag
	 * @return Bool indicating whether or not WEP is enabled
	 */
	bool enableWEP() const { return _enableWEP; }

	/**
	 * Accessor method for the name of this profile
	 * @return The name of this profile
	 */
	QString name() const { return _name; }
						
	/** Method to set the SSID
	 * @param newSSID The new SSID
	 */
	void setSSID( const QString &newSSID );
    
	/** Method to set the new WEP Key
	 * @param newWEPKey The new WEP Key 
	 */
	void setWEPKey( const QString &newWEPKey );
    
	/** Method to set the new Mode
	 * @param newMode The new mode to set
	 */
	void setMode( int newMode );
    
	/** Method to set the WEP Enabled flag
	 * @param wepEnabled Bool indicating whether or not to use WEP
	 */
	void setWEPEnabled( bool wepEnabled );
		
	/**
	 * Method to set the name of this profile
	 * @param name The name of this profile
	 */
	void setName( const QString &name );

	/**
	 * Method to tell the CardProfile to write itself
	 * out to the config file, it uses it's name attribute
	 * to set the group, so multiple CardProfiles with the
	 * same name will overwrite eachother in the config
	 */
	void writeConfig();

	/**
	 * Method to have the CardConfig load it's values from
	 * the global KConfig object. The name will be set to 
	 * parameter after the configuration has been loaded. 
	 * If no configuration exists for the name specified, no
	 * record will be loaded
	 * @param name The name of the profile to load
	 */
	void loadConfig( const QString &name );
		
	// Mode descriptors for the card
	static const int MODE_ADHOC = 1;
	static const int MODE_INFRASTRUCTURE = 2;
    
protected:
	/** The SSID */
	QString _SSID;
	/** The WEP Key */
	QString _WEPKey;
	/** The mode (either CardProfile::MODE_ADHOC or
	 * CardProfile::MODE_INFRASTRUCTURE)
	 */
	int _Mode;
	/** Bool indicating whether or not to enable WEP */
	bool _enableWEP;
	/** Text name for this profile */
	QString _name;
	/** Pointer to the config object */
	KConfig *_config;
		
};

/** Overloaded == operator for this class */
bool operator==( const CardProfile& a, const CardProfile& b );


#endif
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */
