/*
	 KiFi, KDE Wireless Manager
	 Copyright (c) 2002-2004 by Chris TenHarmsel <chris@staticmethod.net>

	 KiFi, (c) 2004 by the KiFi developers

 *************************************************************************
 *                                                                       *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 *************************************************************************
 */

#include "networkinterface.h"

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <linux/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
// Private copy
#include <iwlib.h>
#include <kdebug.h>

// static variable defs
// TODO: We should do something a little more flexible here.... (chris)
const QString NetworkInterface::ROUTE_CMD("/sbin/route");
const QString NetworkInterface::DHCPCD_CMD("/sbin/dhcpcd");

NetworkInterface::NetworkInterface(const QString & ifname) : _ifname(ifname),
	_configured(false),
	_useDHCP(false),
	_dhcpcdPidFile("/var/run/dhcpcd-"+_ifname+".pid")
{
}

NetworkInterface::~NetworkInterface()
{
}

void NetworkInterface::configureStatic(
		const QString & address,
		const QString & broadcast,
		const QString & netmask,
		const QString & gateway)
{
	_configured = true;
	_useDHCP = false;
	_address = address;
	_broadcast = broadcast;
	_netmask = netmask;
	_gateway = gateway;
}

void NetworkInterface::configureDHCP(const QString & gateway)
{
	_configured = true;
	_useDHCP = true;
	_gateway = gateway;
}

bool NetworkInterface::start()
{
	if (!_configured) {
		return false;
	}

	QString command;

	if (_useDHCP) {
		// exec /sbin/dhcpcd
		command = DHCPCD_CMD + " " + _ifname;
		int retval;
		// TODO: system does not return 0 on good, see man page for system.
		kdDebug() << "Execing: [" << command << "]" << endl;
		if ((retval = system(command.latin1()))) {
			kdDebug() << "Error execing: " << command << ": "
				<< (retval >> 8) << endl;
			return false;
		} else {
			return true;
		}
	} else {
		struct ifreq if_req;
		// TODO this should probably try more address families
		// maybe use iw_sockets_open() instead?
		int sock_fd = socket(AF_INET, SOCK_DGRAM, 0);

		// set ifname as the interface name in
		// the info struct
		strncpy(if_req.ifr_name, _ifname.latin1(), IFNAMSIZ);

		// fill up info struct with current flags
		if (ioctl(sock_fd, SIOCGIFFLAGS, &if_req) < 0) {
			kdDebug() << "Error getting IF flags: "<< _ifname
				<< ": " << strerror(errno) << endl;
			return false;
		}

		// set broadcast
		if (!setAddrOption(sock_fd, SIOCSIFBRDADDR,
					_broadcast, &if_req)) {
			kdDebug() << "Error setting broadcast addr: "
				<< strerror(errno) << endl;
			return false;
		}

		// set netmask
		if (!setAddrOption(sock_fd, SIOCSIFNETMASK,
					_netmask, &if_req)) {
			kdDebug() << "Error setting netmask addr: "
				<< strerror(errno) << endl;
			return false;
		}

		// set address
		if (!setAddrOption(sock_fd, SIOCSIFADDR,
					_address, &if_req)) {
			kdDebug() << "Error setting main address: "
				<< strerror(errno) << endl;
			return false;
		}

		// set the UP and RUNNING flags on ifname
		if_req.ifr_flags |=
			(IFF_UP | IFF_RUNNING | IFF_BROADCAST);
		if (ioctl(sock_fd, SIOCSIFFLAGS, &if_req) < 0) {
			kdDebug() << "Error setting IF " << _ifname
				<< " to UP: " << strerror(errno) << endl;
			return false;
		}
	}

	if (_gateway.length() > 0) {
		int retval;
		//
		// The route code is much more complicated
		// than that of ifconfig. Use system() for now.
		//

		// First delete any existing route if it was
		// setup by kernel...
		command = ROUTE_CMD + " del default dev " + _ifname;
		if ((retval = system(command.latin1()))) {
			kdDebug() << "Error execing: " << command << ": "
				<< retval << endl;
			// this is ok, we're only making sure that no
			// kernel routes are still in place
		}

		// TODO# Second delete old gateway if it was set...
		// /sbin/route del default gw ${ogw} &>${devnull}
		// delete the old one; find it in /proc/net/route

		// # Third add our new default gateway
		// /sbin/route add default gw ${gw} >${devnull}
		command = ROUTE_CMD + " add default gw " + _gateway;
		if ((retval = system(command.latin1()))) {
			kdDebug() << "Error execing: " << command << ": "
				<< retval << endl;
			return false;
		}
	}

	return true;
}

bool NetworkInterface::stop()
{
	kdDebug() << "NetworkInterface::stop entering" << endl;

	if (_useDHCP) {
		// NOTE: this code currently assumes dhcpcd as only
		// possibility of dhcp client tool
		kdDebug() << "releasing DHCP lease" << endl;
		int retval = -1;
		struct stat fData;

		// this command will send SIGHUP to dhcpcd telling it
		// to release the lease and destroy its cache
		QString command = DHCPCD_CMD + " -k " + _ifname;

		for (int i = 0; i < 10; i++) {
			// check for pid file
			if (stat(_dhcpcdPidFile.latin1(), &fData) == 0) {
				kdDebug() << "found dhcpcd pid file, trying to take it down: ["
						<< command << "]" << endl;
				// a process is running, tell it to let go... let go...
				if (!(retval = system(command.latin1()))) {
					kdDebug() << "Success issuing [" << command << "]" << endl;
					// success
					break;
				}
				sleep(1);
			} else {
				kdDebug() << "no dhcpcd pid file found" << endl;
				// no pid file = no process to signal SIGUSR1
				break;
			}
		}
		// did dhcpcd return failure?
		if (retval != 0) {
			// don't give up, we'll take it down below
			kdDebug() << "couldn't stop dhcpcd" << endl;
		}
	} // end if (_useDHCP)

	struct ifreq if_req;
	// TODO this should probably try more address families
	// maybe use iw_sockets_open() instead?
	kdDebug() << "NetworkInterface::stop(): creating socket" << endl;
	int sock_fd = socket(AF_INET, SOCK_DGRAM, 0);

	// set ifname as the interface name in
	// the info struct
	strncpy(if_req.ifr_name, _ifname.latin1(), IFNAMSIZ);

	// fill up info struct with current flags
	if (ioctl(sock_fd, SIOCGIFFLAGS, &if_req) < 0) {
		kdDebug() << "Error getting IF flags: "<< _ifname
			<< ": " << strerror(errno) << endl;
		return false;
	}

	// set ifname as the interface name in
	// the info struct
	strncpy(if_req.ifr_name, _ifname.latin1(), IFNAMSIZ);

	// unset the UP flag on ifname
	if_req.ifr_flags &= ~IFF_UP;
	if (ioctl(sock_fd, SIOCSIFFLAGS, &if_req) < 0) {
		kdDebug() << "Error setting IF " << _ifname
			<< " to DOWN: " << strerror(errno) << endl;
		return false;
	}

	kdDebug() << "NetworkInterface::stop(): FINISHED true" << endl;
	return true;
}

bool NetworkInterface::setAddrOption(int fd,
		int ioctlFlag, const QString & addr,
		struct ifreq * if_req)
{
	struct sockaddr_in s_addr;
	void * to_addr = NULL;

	switch (ioctlFlag)
	{
		case SIOCSIFADDR:
			to_addr = &if_req->ifr_addr;
			break;
		case SIOCSIFBRDADDR:
			to_addr = &if_req->ifr_broadaddr;
			break;
		case SIOCSIFNETMASK:
			to_addr = &if_req->ifr_netmask;
			break;
		default:
			kdDebug() << "Error invalid ioctlFlag: "
				<< ioctlFlag << endl;
			return false;
	}

	if (!inet_aton(addr.latin1(), &(s_addr.sin_addr))) {
		kdDebug() << "Error parsing address: "
			<< strerror(errno) << endl;
		return false;
	}

	memcpy(to_addr, &s_addr, sizeof(struct sockaddr));

	return (ioctl(fd, ioctlFlag, if_req) < 0);
}

/*!
    \fn NetworkInterface::setEssid(const QString &)
 */
bool NetworkInterface::setEssid(const QString &ssid)
{
	char essid[ IW_ESSID_MAX_SIZE + 1 ];
	memset(&essid, '\0', IW_ESSID_MAX_SIZE + 1);

	struct iwreq wrq;

	if(ssid == "any"){
		wrq.u.essid.flags = 0;
	} else {
		if(ssid.length() > IW_ESSID_MAX_SIZE){
			kdDebug() << "Requested ESSID too long" << endl;
			return false;
		} else {
			wrq.u.essid.flags = 1;
			qstrncpy( essid, ssid.latin1(), IW_ESSID_MAX_SIZE );
			wrq.u.essid.pointer = (caddr_t)essid;
			wrq.u.essid.length = ssid.length() + 1;
		}
	}
	// Open a socket
	int sockfd = iw_sockets_open();
	// Call method
	int retval = iw_set_ext(sockfd, _ifname.latin1(), SIOCSIWESSID, &wrq);
	// close socket
	close(sockfd);
	if(retval < 0){
		kdDebug() << "Setting ESSID unsuccessful on interface: " << _ifname << endl;
		return false;
	} else {
		kdDebug() << "Setting ESSID successful on interface: " << _ifname << endl;
		return true;
	}
}


/*!
    \fn NetworkInterface::setWepKey(const QString &)
 */
bool NetworkInterface::setWepRequired(const QString &wepKey)
{
	struct iwreq wrq;
	wrq.u.data.pointer = ( caddr_t ) NULL;
	wrq.u.data.flags = 0;
	wrq.u.data.length = 0;

	// Create the key structure
	unsigned char	key[ IW_ENCODING_TOKEN_MAX ];
	// Open a socket
	int sockfd = iw_sockets_open();

	// Parse the key
	int keySize = iw_in_key_full(sockfd, _ifname.latin1(),
			wepKey.latin1(), key, &wrq.u.data.flags);
	// Set it if it's got a length
	if(keySize > 0){
		wrq.u.data.length = keySize;
		wrq.u.data.pointer = ( caddr_t ) key;
	}

	if(wrq.u.data.pointer == 0){
		wrq.u.data.flags |= IW_ENCODE_NOKEY;
	}

	int retval = iw_set_ext(sockfd, _ifname.latin1(), SIOCSIWENCODE, &wrq);
	bool ourReturn = false;
	if(retval < 0){
		kdDebug() << "Setting WEP key not successful on device: " << _ifname << endl;
		ourReturn = false;
	} else {
		kdDebug() << "Setting WEP key successful on device: " << _ifname << endl;
		ourReturn = true;
	}
	// Close the socket
	close(sockfd);
	return ourReturn;
}


/*!
    \fn NetworkInterface::setWepRequired(bool)
 */
bool NetworkInterface::setWepNotRequired()
{
	kdDebug() << "NetworkInterface::setWepNotRequired(): ENTER" << endl;
	struct iwreq wrq;
	wrq.u.data.pointer = ( caddr_t ) NULL;
	wrq.u.data.flags = 0;
	wrq.u.data.length = 0;
	wrq.u.data.flags |= IW_ENCODE_DISABLED;

	// Open a socket
	int sockfd = iw_sockets_open();

	int retval = iw_set_ext(sockfd, _ifname.latin1(), SIOCSIWENCODE, &wrq);
	close(sockfd);
	if(retval < 0){
		kdDebug() << "Could not disable encoding" << endl;
		return false;
	} else {
		kdDebug() << "Encoding disabled successfully" << endl;
		return true;
	}
}

/*!
    \fn NetworkInterface::setMode(const QString &)
 */
bool NetworkInterface::setMode(const QString &mode)
{
	// See if we can find the specified mode in the allowed
	// modes
	kdDebug() << "Checking the mode value passed in: ["
			<< mode << "]" << endl;
	int modeIndex = -1;
	for(int i = 0; i < IW_NUM_OPER_MODE; i++) {
		// All the first three letters are different
		if( mode == iw_operation_mode[i] ) {
			modeIndex = i;
			break;
		}
	}

	// If we have a good modes
	if(modeIndex != -1) {
		kdDebug() << "Requested mode valid" << endl;
		struct iwreq wrq;
		wrq.u.mode = modeIndex;
		// Open a socket
		int sockfd = iw_sockets_open();

		// Return value
		bool retval;

		// Make the call
		if(iw_set_ext(sockfd, _ifname.latin1(), SIOCSIWMODE, &wrq) < 0) {
			kdDebug() << "Setting the Mode failed on device: " << _ifname
					<< " [" << strerror(errno) << "]" << endl;
			retval = false;
		} else {
			kdDebug() << "Setting mode successful on device: " << _ifname << endl;
			retval = true;
		}
		// Close the socket
		close(sockfd);
		// return
		return retval;
	} else {
		kdDebug() << "Bad mode requested" << endl;
		return false;
	}
}

/*!
    \fn NetworkInterface::commitWirelessInfo()
 */
bool NetworkInterface::commitWirelessInfo()
{
	int sockfd = iw_sockets_open();
	struct iwreq wrq;
	int retval = iw_set_ext(sockfd, _ifname.latin1(), SIOCSIWCOMMIT, &wrq);
	close(sockfd);
	if(retval < 0){
		return false;
	} else {
		return true;
	}
}

/*!
	\fn NetworkInterface::up()
 */
bool NetworkInterface::up()
{
  struct ifreq if_req;
	// TODO this should probably try more address families
	// maybe use iw_sockets_open() instead?
	int sock_fd = socket(AF_INET, SOCK_DGRAM, 0);

	// set ifname as the interface name in
	// the info struct
	strncpy(if_req.ifr_name, _ifname.latin1(), IFNAMSIZ);

		// fill up info struct with current flags
	if (ioctl(sock_fd, SIOCGIFFLAGS, &if_req) < 0) {
		kdDebug() << "Error getting IF flags: "<< _ifname
			<< ": " << strerror(errno) << endl;
		return false;
	}

	// set the UP and RUNNING flags on ifname
	if_req.ifr_flags |= IFF_UP;
	if (ioctl(sock_fd, SIOCSIFFLAGS, &if_req) < 0) {
		kdDebug() << "Error setting IF " << _ifname
			<< " to UP: " << strerror(errno) << endl;
		return false;
	}

	return true;
}

/*!
	\fn NetworkInterface::isUp()
 */
	bool NetworkInterface::isUp()
{
	struct ifreq if_req;
	// TODO this should probably try more address families
	// maybe use iw_sockets_open() instead?
	int sock_fd = socket(AF_INET, SOCK_DGRAM, 0);

	// set ifname as the interface name in
	// the info struct
	strncpy(if_req.ifr_name, _ifname.latin1(), IFNAMSIZ);

	if(ioctl(sock_fd, SIOCGIFFLAGS, &if_req) < 0) {
		kdDebug() << "Error getting IF flags: " << _ifname
			<< " : " << strerror(errno) << endl;
		return false;
	}

	// By now we should have the struct filled with info
	short isUp = if_req.ifr_flags & IFF_UP;
	if(isUp == 0) {
		return false;
	} else {
		return true;
	}
}

/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 2
 * indent-tabs-mode: t
 * End:
 */
