/*
    KiFi, KDE Wireless Manager
    Copyright (c) 2002-2004 by Micah Alles <micah@atomicobject.com>

    KiFi, (c) 2004 by the KiFi developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef _NETWORKINTERFACE_H_
#define _NETWORKINTERFACE_H_

#include <qstring.h>

/**
 * This class represents a network interface (e.g. wlan0) and provides
 * methods for starting (statically or w/dhcp) and stoping it.
 */
class NetworkInterface
{
public:
	/** Constructor */
	NetworkInterface(const QString & ifname);

	/** Destructor */
	virtual ~NetworkInterface();

	/**
	 * Set this interface to use a static ip configuration
	 * when it starts.
	 *
	 * @param address ipv4 ip desired address as a string
	 * @param broadcast ipv4 ip broadcast address as a string
	 * @param netmask ipv4 ip netmask address as a string
	 * @param gateway ipv4 ip default gateway address as a string
	 */
	void configureStatic(const QString & address,
			const QString & broadcast,
			const QString & netmask,
			const QString & gateway = "");

	/**
	 * Set this interface to use a dynamic ip configuration
	 * using dhcp.
	 *
	 * @param gateway ipv4 ip default gateway address as a string
	 */
	void configureDHCP(const QString & gateway = "");

	/**
	 * Attempt to start this interface. If not configured
	 * do nothing.
	 *
	 * @return TRUE if successfully started, FALSE if an
	 * error occurs, interface could not be started, or
	 * if not yet configured.
	 */
	bool start();

	/**
	 * Attempt to stop this interface. If already stopped
	 * do nothing.
	 *
	 * @return TRUE if successfully stopped or if already
	 * stopped, FALSE if an error occurs or unable to stop.
	 */
	bool stop();

	/**
	 * This method simply brings this interface up.  The reason for this is that
	 * wireless interfaces can't scan for available networks when they are down
	 * so at the very least they need to be brought up in the system to scan.
	 *
	 * @return TRUE if successfully brought up, FALSE if an error occurs.
	 */
	bool up();

	/**
	 * Sets the ESSID for this interface
	 * @param ssid The SSID to set
	 */
	bool setEssid(const QString &ssid);

	/**
	 * Sets the WEP key the interface should use.
	 * @param wepKey The WEP key
	 */
	bool setWepRequired(const QString &wepKey);

	/**
	 * Sets the card into "open" mode, ie. no WEP key.
	 */
	bool setWepNotRequired();

	/**
	 * Sets the mode the card operates in.
	 * @param mode The mode
	 */
	bool setMode(const QString &mode);

	/**
	 * Commits the info to the card, should be called
	 * after calling the other methods.
	 */
	bool commitWirelessInfo();

	/**
	 * This method should be used to find out if the interface is currently up.
	 * It does not used cached data, but rather queries the system for interface
	 * status.
	 *
	 * @return TRUE if the interface is currently up, FALSE if it is down.
	 */
  bool isUp();

private:
	/**
	 * Utility for setting an address field option on an ifreq struct.
	 */
	bool setAddrOption(int fd, int ioctlFlag,
		const QString & addr, struct ifreq * if_req);



protected:
	/** Name of this interface */
	const QString _ifname;
	/** Whether or not this interface has been configured */
	bool _configured;
	/** Whether or not this interface uses dhcp */
	bool _useDHCP;
	/** Path to interface-specific pid file for DHCPCD */
	QString _dhcpcdPidFile;
	/** Main addr for static configuration */
	QString _address;
	/** Broadcast addr for static configuration */
	QString _broadcast;
	/** Netmask addr for static configuration */
	QString _netmask;
	/** Gateway addr */
	QString _gateway;

	/** Full path to route command */
	static const QString ROUTE_CMD;
	/** Full path to dhcpcd command */
	static const QString DHCPCD_CMD;
};


#endif
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 2
 * indent-tabs-mode: t
 * End:
 */
