/*
KiFi, KDE Wireless Manager
Copyright (c) 2002-2004 by Chris TenHarmsel <chris@staticmethod.net>

KiFi, (c) 2004 by the KiFi developers

*************************************************************************
*                                                                       *
* This program is free software; you can redistribute it and/or modify  *
* it under the terms of the GNU General Public License as published by  *
* the Free Software Foundation; either version 2 of the License, or     *
* (at your option) any later version.                                   *
*                                                                       *
*************************************************************************
*/

#ifndef _WIRELESSNETWORKINFO_H_
#define _WIRELESSNETWORKINFO_H_

#include <qstring.h>
#include <qstringlist.h>

/**
 * This class represents information about a network.  Objects of this
 * type are provided by the WirelessNetworkScanner class when it discovers
 * available wireless networks
 *
 * @author Chris TenHarmsel <chris@staticmethod.net>
 */
class WirelessNetworkInfo
{
public:
	/** Default Constructor */
	WirelessNetworkInfo();

	/** Constructor */
	WirelessNetworkInfo(
		QString & address, int channel, const QString &freq, QString &mode,
		int nwid, bool nwidEnabled, int signalStrength, QString & ssid,
		QStringList supportedRates, bool wepRequired );

	/** Copy constructor */
	WirelessNetworkInfo( const WirelessNetworkInfo &oldInfo );

	/**
	 * Gets the address of this network
	 * @return the address of this network
	 */
	QString getAddress();

	/**
	 * Gets the SSID of this network
	 * @return the SSID of this network
	 */
	QString getSSID();

	/**
	 * Gets whether or not WEP is required
	 * @return boolean indicating whether or not WEP is required on this network
	 */
	bool getWepRequired();

	/**
	 * Gets the signal strength of this network
	 * @return the signal strength of this network
	 */
	int getSignalStrength();

	/**
	 * This method sets the address in this information holder.
	 * @param newAddress The new address to set.
	 */
	void setAddress( const QString &newAddress );

	/**
	 * This method sets the ssid in this information holder.
	 * @param newSSID The new SSID to set.
	 */
	void setSSID( const QString &newSSID );

	/**
	 * This method sets whether or not this network requires WEP.
	 * @param required Whether or not wep is required on this network.
	 */
	void setWepRequired( bool required );

	/**
	 * This method sets new signal strength for the network
	 * @param newQuality The new signal strength
	 */
	void setSignalStrength( int newQuality );

	/**
	 * This method returns whether or not this AP has NWID enabled
	 * @return Bool indicating whether or not NWID is enabled
	 */
	bool getNWIDEnabled();

	/**
	 * Sets whether or not NWID is enabled on this AP
	 * @param flag Whether or not the NWID is enabled
	 */
	void setNWIDEnabled(bool flag);

	/**
	 * Gets the Frequency that this AP operates on.
	 * @return The frequency.
	 */
	QString getFrequency();

	/**
	 * Sets the frequency.
	 * @param newFreq The new frequency that you want to set.
	 */
	void setFrequency(const QString &newFreq);

	/**
	 * Gets the channel this AP operates on.
	 * @return The channel
	 */
	int getChannel();

	/**
	 * Sets the channel that this AP operates on.
	 * @param newChannel The channel.
	 */
	void setChannel(int newChannel);

	/**
	 * Gets the NWID for this AP
	 * @return The AP's NWID
	 */
	int getNWID();

	/**
	 * Sets the NWID for this AP
	 * @param newNwid the new NWID
	 */
	void setNWID(int newNwid);

	/**
	 * Gets the Mode that this AP operates in
	 * @return The Mode
	 */
	QString getMode();

	/**
	 * Sets the mode that this AP Operates in
	 * @para newMode The mode to set
	 */
	void setMode(QString newMode);

	/**
	 * Gets a list of the supported rates
	 * @return a list of QStrings describing the supported rates.
	 */
	QStringList getSupportedRates();

	/**
	 * Sets supported rates
	 * @param newRates Sets the rates supported by this AP
	 */
	void setSupportedRates(QStringList newRates);

	/**
	 * Adds a supported rate to the list of rates
	 * @param rate The rate to add
	 */
	void addSupportedRate(QString newRate);

	/** Assignment operator */
	WirelessNetworkInfo &operator=( const WirelessNetworkInfo &oldInfo );

	/** Equals operator */
	bool operator==( const WirelessNetworkInfo &otherInfo );
	/** Not equals operator */
	//bool operator!=( const WirelessNetworkInfo &otherInfo );

protected:
	/** The address of this network */
	QString _address;
	/** The channel */
	int _channel;
	/** The frequency */
	QString _freq;
	/** The mode, as a string */
	QString _mode;
	/** The NWID of the AP */
	int _nwid;
	/** Flag indicating whether or not the NWID is enabled on this AP */
	bool _nwidEnabled;
	/** The signal strength of the network */
	int _signalStrength;
	/** The SSID of this network */
	QString _ssid;
	/** A list of string representations of supported bit rates */
	QStringList _supportedRates;
	/** Flag indicating whether or not WEP is required on this network */
	bool _wepRequired;

};

#endif
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 2
 * indent-tabs-mode: t
 * End:
 */
